@@ -65,13 +65,13 @@ typedef enum {
 
 typedef enum {
 	PDF_GRAPHIC_MODE_GRAPHICS,
-	PDF_GRAPHIC_MODE_TEXT,
+	PDF_GRAPHIC_MODE_TEXT
 } PdfGraphicMode;
 
 typedef enum {
 	PDF_COLOR_MODE_DEVICEGRAY,
 	PDF_COLOR_MODE_DEVICERGB,
-	PDF_COLOR_MODE_DEVICECMYK,
+	PDF_COLOR_MODE_DEVICECMYK
 }PdfColorModes;
 
 typedef enum {
@@ -517,22 +517,22 @@ gnome_print_pdf_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 	else
 		page->used_color_images = TRUE;
 	gnome_print_pdf_page_fprintf (pdf, "q" EOL);
-	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+	gnome_print_pdf_page_print_double (pdf, "%g",
 				      ctm[0]);
 	gnome_print_pdf_page_fprintf (pdf, " ");
-	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+	gnome_print_pdf_page_print_double (pdf, "%g",
 				      ctm[1]);
 	gnome_print_pdf_page_fprintf (pdf, " ");
-	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+	gnome_print_pdf_page_print_double (pdf, "%g",
 				      ctm[2]);
 	gnome_print_pdf_page_fprintf (pdf, " ");
-	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+	gnome_print_pdf_page_print_double (pdf, "%g",
 				      ctm[3]);
 	gnome_print_pdf_page_fprintf (pdf, " ");
-	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+	gnome_print_pdf_page_print_double (pdf, "%g",
 				      ctm[4]);
 	gnome_print_pdf_page_fprintf (pdf, " ");
-	gnome_print_pdf_page_print_double (pdf, "%g" EOL,
+	gnome_print_pdf_page_print_double (pdf, "%g",
 				      ctm[5]);
 	gnome_print_pdf_page_fprintf (pdf, " cm" EOL);
 	gnome_print_pdf_page_fprintf (pdf, "0 0 m" EOL);
@@ -1688,9 +1688,9 @@ gnome_print_pdf_print_bpath (GnomePrintPdf *pdf, const ArtBpath *bpath)
 				gnome_print_pdf_page_fprintf (pdf, "h" EOL);
 			closed = FALSE;
 			started = FALSE;
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x3);
 			gnome_print_pdf_page_fprintf (pdf, " ");
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y3);
 			gnome_print_pdf_page_fprintf (pdf, " m" EOL);
 			break;
 		case ART_MOVETO:
@@ -1698,29 +1698,29 @@ gnome_print_pdf_print_bpath (GnomePrintPdf *pdf, const ArtBpath *bpath)
 				gnome_print_pdf_page_fprintf (pdf, "h" EOL);
 			closed = TRUE;
 			started = TRUE;
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x3);
 			gnome_print_pdf_page_fprintf (pdf, " ");
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y3);
 			gnome_print_pdf_page_fprintf (pdf, " m" EOL);
 			break;
 		case ART_LINETO:
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x3);
 			gnome_print_pdf_page_fprintf (pdf, " ");
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y3);
 			gnome_print_pdf_page_fprintf (pdf, " l" EOL);
 			break;
 		case ART_CURVETO:
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x1);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x1);
 			gnome_print_pdf_page_fprintf (pdf, " ");
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y1);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y1);
 			gnome_print_pdf_page_fprintf (pdf, " ");
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x2);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x2);
 			gnome_print_pdf_page_fprintf (pdf, " ");
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y2);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y2);
 			gnome_print_pdf_page_fprintf (pdf, " ");
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->x3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->x3);
 			gnome_print_pdf_page_fprintf (pdf, " ");
-			gnome_print_pdf_page_print_double (pdf, "%g" EOL, bpath->y3);
+			gnome_print_pdf_page_print_double (pdf, "%g", bpath->y3);
 			gnome_print_pdf_page_fprintf (pdf, " c" EOL);
 			break;
 		default:
