@@ -20,16 +20,14 @@
  *    Chema Celorio <chema@celorio.com>
  *    Lauris Kaplinski <lauris@helixcode.com>
  *
- *  Copyright 2000-2001 Ximian, Inc. and authors
+ *  Copyright 2000-2003 Ximian, Inc.
  *
  *  References:
- *    Document Structuring Conventions, Adobe. [http://partners.adobe.com/asn/developer/pdfs/tn/5001.DSC_Spec.pdf]
+ *    [1] Postscript Language Reference, 3rd Edition. Adobe. [http://partners.adobe.com/asn/developer/pdfs/tn/PLRM.pdf]
+ *    [2] Document Structuring Conventions, Adobe. [http://partners.adobe.com/asn/developer/pdfs/tn/5001.DSC_Spec.pdf]
  */
 
-#define __GNOME_PRINT_PS2_C__
-
-#include "config.h"
-
+#include <config.h>
 #include <math.h>
 #include <string.h>
 #include <time.h>
@@ -42,10 +40,9 @@
 #include <libgnomeprint/gp-gc-private.h>
 #include <libgnomeprint/gnome-print-transport.h>
 #include <libgnomeprint/gnome-font-private.h>
-#include <libgnomeprint/gnome-print-encode-private.h>
+#include <libgnomeprint/gnome-print-encode.h>
 #include <libgnomeprint/gnome-pgl-private.h>
-
-#include "gnome-print-ps2.h"
+#include <libgnomeprint/gnome-print-ps2.h>
 
 #define EOL "\n"
 
@@ -65,12 +62,15 @@ struct _GnomePrintPs2 {
 	/* lists */
 	GnomePrintPs2Font *fonts;
 	GnomePrintPs2Page *pages;
+	GHashTable *fonts_hash; /* GnomePrintPS2Font to psname */
 
 	/* State */
 	GnomePrintPs2Font *selected_font;
 	gdouble r, g, b;
 	gint private_color_flag;
 	gint gsave_level;
+
+	gboolean new_copy;
 	
 	/* Buffer */
 	FILE *buf;
@@ -82,6 +82,7 @@ struct _GnomePrintPs2Font {
 	GnomeFontFace *face;
 	GnomeFontPsObject *pso;
 	gdouble currentsize;
+	gint instance; /* See #105063 */
 };
 
 struct _GnomePrintPs2Page {
@@ -106,6 +107,7 @@ static gint gnome_print_ps2_image (GnomePrintContext *pc, const gdouble *affine,
 static gint gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList *gl);
 static gint gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name);
 static gint gnome_print_ps2_showpage (GnomePrintContext *pc);
+static gint gnome_print_ps2_end_doc (GnomePrintContext *pc);
 static gint gnome_print_ps2_close (GnomePrintContext *pc);
 
 static gint gnome_print_ps2_set_color (GnomePrintPs2 *ps2);
@@ -119,6 +121,7 @@ static gint gnome_print_ps2_print_bpath (GnomePrintPs2 *ps2, const ArtBpath *bpa
 
 static gchar *gnome_print_ps2_get_date (void);
 static gint   gnome_print_ps2_fprintf (GnomePrintPs2 *ps2, const char *format, ...);
+static gint   gnome_print_ps2_print_double (GnomePrintPs2 *ps2, const gchar *format, gdouble x);
 
 static GnomePrintContextClass *parent_class;
 
@@ -157,6 +160,7 @@ gnome_print_ps2_class_init (GnomePrintPs2Class *klass)
 	pc_class->construct = gnome_print_ps2_construct;
 	pc_class->beginpage = gnome_print_ps2_beginpage;
 	pc_class->showpage = gnome_print_ps2_showpage;
+	pc_class->end_doc = gnome_print_ps2_end_doc;
 	pc_class->gsave = gnome_print_ps2_gsave;
 	pc_class->grestore = gnome_print_ps2_grestore;
 	pc_class->clip = gnome_print_ps2_clip;
@@ -172,10 +176,12 @@ gnome_print_ps2_init (GnomePrintPs2 *ps2)
 {
 	ps2->gsave_level = 0;
 	ps2->fonts = NULL;
+	ps2->fonts_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	ps2->selected_font = NULL;
 	ps2->private_color_flag = GP_GC_FLAG_UNSET;
 	ps2->pages = NULL;
 	ps2->buf = NULL;
+	ps2->new_copy = FALSE;
 }
 
 static void
@@ -212,6 +218,8 @@ gnome_print_ps2_finalize (GObject *object)
 		g_free (p);
 	}
 
+	g_hash_table_destroy (ps2->fonts_hash);
+
 	while (ps2->fonts) {
 		GnomePrintPs2Font *f;
 		f = ps2->fonts;
@@ -308,7 +316,6 @@ gnome_print_ps2_clip (GnomePrintContext *ctx, const ArtBpath *bpath, ArtWindRule
 
 	ps2 = GNOME_PRINT_PS2 (ctx);
 
-	/* FIXME, do we not have to set the line and color atributes here? (Chema) */
 	ret = gnome_print_ps2_print_bpath (ps2, bpath);
 	g_return_val_if_fail (ret >= 0, ret);
 
@@ -329,7 +336,6 @@ gnome_print_ps2_fill (GnomePrintContext *ctx, const ArtBpath *bpath, ArtWindRule
 
 	ps2 = GNOME_PRINT_PS2 (ctx);
 
-	/* FIXME, do we not have to set the line and color atributes here? (Chema) */
 	ret += gnome_print_ps2_set_color (ps2);
 	ret += gnome_print_ps2_print_bpath (ps2, bpath);
 
@@ -377,8 +383,19 @@ gnome_print_ps2_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 
 	ps2 = GNOME_PRINT_PS2 (pc);
 
-	ret += gnome_print_ps2_fprintf (ps2, "q" EOL);
-	ret += gnome_print_ps2_fprintf (ps2, "[%g %g %g %g %g %g]cm" EOL, ctm[0], ctm[1], ctm[2], ctm[3], ctm[4], ctm[5]);
+	ret += gnome_print_ps2_fprintf (ps2, "q" EOL "[");
+	ret += gnome_print_ps2_print_double (ps2, "%g", ctm[0]);
+	ret += gnome_print_ps2_fprintf (ps2, " ");
+	ret += gnome_print_ps2_print_double (ps2, "%g", ctm[1]);
+	ret += gnome_print_ps2_fprintf (ps2, " ");
+	ret += gnome_print_ps2_print_double (ps2, "%g", ctm[2]);
+	ret += gnome_print_ps2_fprintf (ps2, " ");
+	ret += gnome_print_ps2_print_double (ps2, "%g", ctm[3]);
+	ret += gnome_print_ps2_fprintf (ps2, " ");
+	ret += gnome_print_ps2_print_double (ps2, "%g", ctm[4]);
+	ret += gnome_print_ps2_fprintf (ps2, " ");
+	ret += gnome_print_ps2_print_double (ps2, "%g", ctm[5]);
+	ret += gnome_print_ps2_fprintf (ps2, "]cm" EOL);
 
 	/* Image commands */
 	ret += gnome_print_ps2_fprintf (ps2, "/buf %d string def" EOL "%d %d 8" EOL, w * ch, w, h);
@@ -428,7 +445,21 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 	if (!identity) {
 		ret = gnome_print_ps2_fprintf (ps2, "q" EOL);
 		g_return_val_if_fail (ret >= 0, ret);
-		ret = gnome_print_ps2_fprintf (ps2, "[%g %g %g %g %g %g]cm" EOL, a[0], a[1], a[2], a[3], a[4], a[5]);
+
+		ret  = gnome_print_ps2_fprintf (ps2, "[");
+		ret += gnome_print_ps2_print_double (ps2, "%g", a[0]);
+		ret += gnome_print_ps2_fprintf (ps2, " ");
+		ret += gnome_print_ps2_print_double (ps2, "%g", a[1]);
+		ret += gnome_print_ps2_fprintf (ps2, " ");
+		ret += gnome_print_ps2_print_double (ps2, "%g", a[2]);
+		ret += gnome_print_ps2_fprintf (ps2, " ");
+		ret += gnome_print_ps2_print_double (ps2, "%g", a[3]);
+		ret += gnome_print_ps2_fprintf (ps2, " ");
+		ret += gnome_print_ps2_print_double (ps2, "%g", a[4]);
+		ret += gnome_print_ps2_fprintf (ps2, " ");
+		ret += gnome_print_ps2_print_double (ps2, "%g", a[5]);
+		ret += gnome_print_ps2_fprintf (ps2, "]cm" EOL);
+
 		g_return_val_if_fail (ret >= 0, ret);
 		dx = dy = 0.0;
 	} else {
@@ -451,12 +482,17 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 						((ps->color >> 16) & 0xff) / 255.0,
 						((ps->color >>  8) & 0xff) / 255.0);
 		g_return_val_if_fail (ret >= 0, ret);
-		ret = gnome_print_ps2_fprintf (ps2, "%g %g m" EOL, pgl->glyphs[ps->start].x + dx, pgl->glyphs[ps->start].y + dy);
+
+		ret  = gnome_print_ps2_print_double 
+			(ps2, "%g", pgl->glyphs[ps->start].x + dx);
+		ret += gnome_print_ps2_fprintf (ps2, " ");
+		ret += gnome_print_ps2_print_double 
+			(ps2, "%g", pgl->glyphs[ps->start].y + dy);
+		ret += gnome_print_ps2_fprintf (ps2, " m" EOL);
+
 		g_return_val_if_fail (ret >= 0, ret);
 		/* Build string */
 
-
-
 		ret = gnome_print_ps2_fprintf (ps2, "(");
 
 		if (ps2->selected_font->pso->encodedbytes == 1) {
@@ -485,9 +521,14 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 		ret = gnome_print_ps2_fprintf (ps2, "[");
 		g_return_val_if_fail (ret >= 0, ret);
 		for (i = ps->start + 1; i < ps->start + ps->length; i++) {
-			ret = gnome_print_ps2_fprintf (ps2, "%g %g ",
-					      pgl->glyphs[i].x - pgl->glyphs[i-1].x,
-					      pgl->glyphs[i].y - pgl->glyphs[i-1].y);
+			ret = gnome_print_ps2_print_double 
+				(ps2, "%g", 
+				 pgl->glyphs[i].x - pgl->glyphs[i-1].x);
+			ret += gnome_print_ps2_fprintf (ps2, " ");
+			ret += gnome_print_ps2_print_double
+				(ps2, "%g",
+				 pgl->glyphs[i].y - pgl->glyphs[i-1].y);
+			ret += gnome_print_ps2_fprintf (ps2, " ");
 			g_return_val_if_fail (ret >= 0, ret);
 		}
 		ret = gnome_print_ps2_fprintf (ps2, "0 0] ");
@@ -509,6 +550,125 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 	return GNOME_PRINT_OK;
 }
 
+static void
+gnome_print_ps2_setpagedevice_policies (GnomePrintContext *pc)
+{
+	GnomePrintPs2 *ps2 = GNOME_PRINT_PS2 (pc);
+	gboolean duplex = FALSE;
+	gboolean tumble = FALSE;
+	gboolean collate = FALSE;
+	int numcopies = 1;
+	
+	gint x = (ps2->bbox.x1 - ps2->bbox.x0);
+	gint y = (ps2->bbox.y1 - ps2->bbox.y0);
+
+	gnome_print_config_get_int (pc->config, 
+				    GNOME_PRINT_KEY_NUM_COPIES,
+				    &numcopies);
+	gnome_print_config_get_boolean (pc->config, 
+					GNOME_PRINT_KEY_COLLATE, 
+					&collate);
+	gnome_print_config_get_boolean (pc->config, 
+					GNOME_PRINT_KEY_DUPLEX, 
+					&duplex);
+	gnome_print_config_get_boolean (pc->config, 
+					GNOME_PRINT_KEY_TUMBLE, 
+					&tumble);
+	if (collate)
+		numcopies = 1;
+
+	gnome_print_transport_printf (pc->transport, "<<" EOL);
+	gnome_print_transport_printf (pc->transport, "/PageSize [%d %d]" EOL,
+				      x, y);
+	gnome_print_transport_printf (pc->transport, "/ImagingBBox null" EOL);
+	gnome_print_transport_printf (pc->transport, "/Duplex %s" EOL, 
+				      duplex ? "true" : "false");
+	gnome_print_transport_printf (pc->transport, "/Tumble %s" EOL, 
+				      tumble ? "true" : "false");
+	gnome_print_transport_printf (pc->transport, 
+				      "/NumCopies %i" EOL, numcopies);
+	gnome_print_transport_printf (pc->transport, "/Policies <<" EOL);
+	gnome_print_transport_printf (pc->transport, "/PolicyNotFound 1" EOL);
+	gnome_print_transport_printf (pc->transport, "/PageSize 3" EOL);
+	gnome_print_transport_printf (pc->transport, ">>" EOL);
+	gnome_print_transport_printf (pc->transport, ">> setpagedevice" EOL);
+}
+
+static void
+gnome_print_ps2_requirements (GnomePrintContext *pc)
+{
+	gboolean duplex = FALSE;
+	gboolean tumble = FALSE;
+	gboolean collate = FALSE;
+	int numcopies = 1;
+	
+	gnome_print_config_get_int (pc->config, 
+				    GNOME_PRINT_KEY_NUM_COPIES,
+				    &numcopies);
+	gnome_print_config_get_boolean (pc->config, 
+					GNOME_PRINT_KEY_COLLATE, 
+					&collate);
+	gnome_print_config_get_boolean (pc->config, 
+					GNOME_PRINT_KEY_DUPLEX, 
+					&duplex);
+	gnome_print_config_get_boolean (pc->config, 
+					GNOME_PRINT_KEY_TUMBLE, 
+					&tumble);
+	if (collate)
+		numcopies = 1;
+
+
+	gnome_print_transport_printf (pc->transport, 
+				      "%%%%Requirements: numcopies(%d)", 
+				      numcopies);
+	if (collate)
+		gnome_print_transport_printf (pc->transport," collate");
+	if (duplex) {
+		if (tumble)
+			gnome_print_transport_printf (pc->transport,
+						      " duplex(tumble)");
+		else
+			gnome_print_transport_printf (pc->transport,
+						      " duplex");
+	}
+	gnome_print_transport_printf (pc->transport, EOL);
+}
+
+
+static void
+gnome_print_ps2_setpagedevice (GnomePrintContext *pc, gboolean allow_duplex)
+{
+	GnomePrintPs2 *ps2 = GNOME_PRINT_PS2 (pc);
+	gboolean duplex = FALSE;
+	gboolean tumble = FALSE;
+
+	if (allow_duplex) {
+		gnome_print_config_get_boolean (pc->config, 
+						GNOME_PRINT_KEY_DUPLEX, 
+						&duplex);
+		gnome_print_config_get_boolean (pc->config, 
+						GNOME_PRINT_KEY_TUMBLE, 
+						&tumble);
+	}
+	
+	gnome_print_ps2_fprintf (ps2, "<<" EOL);
+	gnome_print_ps2_fprintf (ps2, "/Duplex %s" EOL, 
+				 duplex ? "true" : "false");
+	gnome_print_ps2_fprintf (ps2, "/Tumble %s" EOL, 
+				 tumble ? "true" : "false");
+	
+	gnome_print_ps2_fprintf (ps2, ">> setpagedevice" EOL);
+}
+
+static gint 
+gnome_print_ps2_end_doc (GnomePrintContext *pc)
+{
+	GnomePrintPs2 *ps2 = GNOME_PRINT_PS2 (pc);
+	ps2->new_copy = TRUE;
+
+	return GNOME_PRINT_OK;
+}
+
 static gint
 gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 {
@@ -533,9 +693,24 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 	ps2->selected_font = NULL;
 	ps2->private_color_flag = GP_GC_FLAG_UNSET;
 
-	ret += gnome_print_ps2_fprintf (ps2, "%%%%Page: %s %d" EOL, name, page->number);
+	ret += gnome_print_ps2_fprintf (ps2, "%%%%Page: %s %d" EOL, name, 
+					page->number);
 	ret += gnome_print_ps2_fprintf (ps2, "%%%%PageResources: (atend)" EOL);
-
+	
+	if (ps2->new_copy) {
+		gboolean duplex = FALSE;
+		gnome_print_config_get_boolean (pc->config, 
+						GNOME_PRINT_KEY_DUPLEX, 
+						&duplex);
+		
+		if (duplex) {
+			gnome_print_ps2_fprintf (ps2, "%%%%BeginPageSetup" EOL);
+			gnome_print_ps2_setpagedevice (pc, FALSE);
+			gnome_print_ps2_setpagedevice (pc, TRUE);
+			gnome_print_ps2_fprintf (ps2, "%%%%EndPageSetup" EOL);
+		}
+		ps2->new_copy = FALSE;
+	}
 	ret += gnome_print_newpath (pc);
 	ret += gnome_print_moveto (pc, 0.0, 0.0);
 	ret += gnome_print_lineto (pc, ps2->bbox.x1, 0.0);
@@ -573,7 +748,7 @@ gnome_print_ps2_showpage (GnomePrintContext *pc)
 	while (ps2->pages->usedfonts) {
 		GnomePrintPs2Font *font;
 		font = ps2->pages->usedfonts->data;
-		ret += gnome_print_ps2_fprintf (ps2, "%%%%+ font %s" EOL, font->pso->encodedname);
+		ret += gnome_print_ps2_fprintf (ps2, "%%%%+ font (%s) cvn" EOL, font->pso->encodedname);
 		ps2->pages->usedfonts = g_slist_remove (ps2->pages->usedfonts, ps2->pages->usedfonts->data);
 	}
 
@@ -603,19 +778,16 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	}
 
 	/* Do header */
-	/* Comments */
 	date = gnome_print_ps2_get_date ();
-	gnome_print_transport_printf (pc->transport, "%%!PS-Adobe-3.0" EOL);
-	/* fixme: %%BoundingBox: */
+	gnome_print_transport_printf (pc->transport, "%%!PS-Adobe-3.0" EOL);	
+	/* DSC Comments */
 	gnome_print_transport_printf (pc->transport, "%%%%Creator: Gnome Print Version %s" EOL, VERSION);
 	gnome_print_transport_printf (pc->transport, "%%%%CreationDate: %s" EOL, date);
-	/* fixme: %%DocumentData: */
-	/* fixme: Should we use %%Extensions: and drop LanguageLevel? */
 	gnome_print_transport_printf (pc->transport, "%%%%LanguageLevel: 2" EOL);
 	gnome_print_transport_printf (pc->transport, "%%%%DocumentMedia: Regular %d %d 0 () ()" EOL,
 				      (gint) (ps2->bbox.x1 - ps2->bbox.x0), (gint) (ps2->bbox.y1 - ps2->bbox.y0));
 
-	/* Orientation */
+	/* Orientation ([2] Page 43) */
 	gnome_print_transport_printf (pc->transport, "%%%%Orientation: ");
 	orientation = gnome_print_config_get (pc->config, GNOME_PRINT_KEY_ORIENTATION);
 	if (!orientation || strcmp ("R0", orientation) == 0) {
@@ -623,11 +795,11 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	} else if (strcmp ("R90", orientation) == 0) {
 		gnome_print_transport_printf (pc->transport, "Landscape");
 	} else if (strcmp ("R180", orientation) == 0) {
-		gnome_print_transport_printf (pc->transport, "Reverse Portrait");
+		gnome_print_transport_printf (pc->transport, "Portrait");
 	} else if (strcmp ("R270", orientation) == 0) {
-		gnome_print_transport_printf (pc->transport, "Reverse Landscape");
+		gnome_print_transport_printf (pc->transport, "Landscape");
 	} else {
-		g_warning ("Could not interpret Orientation from GnomePrintConfig [%s]<-\n", orientation);
+		g_warning ("Could not interpret Orientation from GnomePrintConfig [%s]\n", orientation);
 	}
 	g_free (orientation);
 	gnome_print_transport_printf (pc->transport, EOL);
@@ -650,8 +822,9 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	gnome_print_transport_printf (pc->transport, "%%%%DocumentSuppliedResources: procset pnome-print-procs-%s" EOL, VERSION);
 	/* %%DocumentSuppliedResources: */
 	for (font = ps2->fonts; font != NULL; font = font->next) {
-		gnome_print_transport_printf (pc->transport, "%%%%+ font %s" EOL, font->pso->encodedname);
+		gnome_print_transport_printf (pc->transport, "%%%%+ font (%s) cvn" EOL, font->pso->encodedname);
 	}
+	gnome_print_ps2_requirements (pc);
 	gnome_print_transport_printf (pc->transport, "%%%%EndComments" EOL);
 	g_free (date);
 	/* Defaults */
@@ -679,15 +852,12 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	/* Setup */
 	gnome_print_transport_printf (pc->transport, "%%%%BeginSetup" EOL);
 	/* setpagedevice*/
-	gnome_print_transport_printf (pc->transport, "<<" EOL);
-	gnome_print_transport_printf (pc->transport, "/PageSize [%d %d]" EOL, (gint) (ps2->bbox.x1 - ps2->bbox.x0), (gint) (ps2->bbox.y1 - ps2->bbox.y0));
-	gnome_print_transport_printf (pc->transport, "/ImagingBBox null" EOL);
-	gnome_print_transport_printf (pc->transport, "/Orientation %d" EOL, 0); /* FIXME: real orientation */
-	gnome_print_transport_printf (pc->transport, ">> setpagedevice" EOL);
+	gnome_print_ps2_setpagedevice_policies (pc);
+
 	/* Download fonts */
 	for (font = ps2->fonts; font != NULL; font = font->next) {
 		gnome_font_face_ps_embed (font->pso);
-		gnome_print_transport_printf (pc->transport, "%%%%BeginResource: font %s" EOL, font->pso->encodedname);
+		gnome_print_transport_printf (pc->transport, "%%%%BeginResource: font (%s) cvn" EOL, font->pso->encodedname);
 		gnome_print_transport_write (pc->transport, font->pso->buf, font->pso->length);
 		gnome_print_transport_printf (pc->transport, "%%%%EndResource" EOL);
 	}
@@ -750,11 +920,15 @@ gnome_print_ps2_set_line (GnomePrintPs2 *ps2)
 	if (gp_gc_get_line_flag (ctx->gc) == GP_GC_FLAG_CLEAR)
 		return 0;
 
-	ret = gnome_print_ps2_fprintf (ps2, "%g w %i J %i j %g M" EOL,
-				       gp_gc_get_linewidth (ctx->gc),
-				       gp_gc_get_linecap (ctx->gc),
-				       gp_gc_get_linejoin (ctx->gc),
-				       gp_gc_get_miterlimit (ctx->gc));
+	ret = gnome_print_ps2_print_double (ps2, "%g", 
+					    gp_gc_get_linewidth (ctx->gc));
+	ret += gnome_print_ps2_fprintf (ps2, " ");
+	ret += gnome_print_ps2_fprintf (ps2, "w %i J %i j ",
+					gp_gc_get_linecap (ctx->gc),
+					gp_gc_get_linejoin (ctx->gc));
+	ret += gnome_print_ps2_print_double (ps2, "%g", 
+					     gp_gc_get_miterlimit (ctx->gc));
+	ret += gnome_print_ps2_fprintf (ps2, " M" EOL);
 	gp_gc_set_line_flag (ctx->gc, GP_GC_FLAG_CLEAR);
 
 	return ret;
@@ -774,9 +948,15 @@ gnome_print_ps2_set_dash (GnomePrintPs2 *ps2)
 
 	dash = gp_gc_get_dash (ctx->gc);
 	ret += gnome_print_ps2_fprintf (ps2, "[");
-	for (i = 0; i < dash->n_dash; i++)
-		ret += gnome_print_ps2_fprintf (ps2, " %g", dash->dash[i]);
-	ret += gnome_print_ps2_fprintf (ps2, "]%g d" EOL, dash->n_dash > 0 ? dash->offset : 0.0);
+	for (i = 0; i < dash->n_dash; i++) {
+		ret += gnome_print_ps2_fprintf (ps2, " ");
+		ret += gnome_print_ps2_print_double 
+			(ps2, "%g", dash->dash[i]);
+	}
+	ret += gnome_print_ps2_fprintf (ps2, "]");
+	ret += gnome_print_ps2_print_double 
+		(ps2, "%g", dash->n_dash > 0 ? dash->offset : 0.0);
+	ret += gnome_print_ps2_fprintf (ps2, " d" EOL);
 	gp_gc_set_dash_flag (ctx->gc, GP_GC_FLAG_CLEAR);
 
 	g_return_val_if_fail (ret >= 0, ret);
@@ -796,7 +976,12 @@ gnome_print_ps2_set_color_real (GnomePrintPs2 *ps2, gdouble r, gdouble g, gdoubl
 	    && (r == ps2->r) && (g == ps2->g) && (b == ps2->b))
 		return GNOME_PRINT_OK;
 	
-	ret = gnome_print_ps2_fprintf (ps2, "%.3g %.3g %.3g rg" EOL, r, g, b);
+	ret = gnome_print_ps2_print_double (ps2, "%.3g", r);
+	ret += gnome_print_ps2_fprintf (ps2, " ");
+	ret += gnome_print_ps2_print_double (ps2, "%.3g", g);
+	ret += gnome_print_ps2_fprintf (ps2, " ");
+	ret += gnome_print_ps2_print_double (ps2, "%.3g", b);
+	ret += gnome_print_ps2_fprintf (ps2, " rg" EOL);
 
 	ps2->r = r;
 	ps2->g = g;
@@ -815,9 +1000,13 @@ gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font)
 	GnomePrintPs2Font *font;
 	gint ret = 0;
 	GSList *l;
+	gint instance = 0;
 
-	if (ps2->selected_font && (ps2->selected_font->face == gnome_font->face) && (ps2->selected_font->currentsize = gnome_font->size))
+	if ((ps2->selected_font) &&
+	    (ps2->selected_font->face == gnome_font->face) &&
+	    (ps2->selected_font->currentsize == gnome_font->size)) {
 		return 0;
+	}
 
 	face = gnome_font_get_face (gnome_font);
 
@@ -825,15 +1014,28 @@ gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font)
 		if (font->face == face)
 			break;
 	}
+	if (!font) {
+		/* See #105063 */
+		GnomePrintPs2Font *clone;
+		clone = g_hash_table_lookup (ps2->fonts_hash, face->psname);
+		if (clone) {
+			instance = clone->instance + 1;
+		}
+	}
 	if (!font) {
 		/* No entry, so create one */
-		font = g_new (GnomePrintPs2Font, 1);
-		font->next = ps2->fonts;
-		ps2->fonts = font;
-		font->face = (GnomeFontFace *) face;
 		gnome_font_face_ref (face);
-		font->pso = gnome_font_face_pso_new ((GnomeFontFace *) face, NULL);
+		
+		font = g_new0 (GnomePrintPs2Font, 1);
+		font->next     = ps2->fonts;
+		font->face     = (GnomeFontFace *) face;
+		font->pso      = gnome_font_face_pso_new ((GnomeFontFace *) face, NULL, instance);	
+		font->instance = instance;
+
 		g_return_val_if_fail (font->pso != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+
+		ps2->fonts = font;
+		g_hash_table_insert (ps2->fonts_hash, face->psname, font);
 	}
 
 	for (l = ps2->pages->usedfonts; l != NULL; l = l->next) {
@@ -844,7 +1046,11 @@ gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font)
 		ps2->pages->usedfonts = g_slist_prepend (ps2->pages->usedfonts, font);
 	}
 
-	ret += gnome_print_ps2_fprintf (ps2, "/%s FF %g F" EOL, font->pso->encodedname, gnome_font_get_size (gnome_font));
+	ret += gnome_print_ps2_fprintf (ps2, "(%s) cvn FF ", 
+					font->pso->encodedname);
+	ret += gnome_print_ps2_print_double (ps2, "%g", 
+					     gnome_font_get_size (gnome_font));
+	ret += gnome_print_ps2_fprintf (ps2, " F" EOL);
 
 	font->currentsize = gnome_font->size;
 	ps2->selected_font = font;
@@ -890,23 +1096,40 @@ gnome_print_ps2_print_bpath (GnomePrintPs2 *ps2, const ArtBpath *bpath)
 				gnome_print_ps2_fprintf (ps2, "h" EOL);
 			closed = FALSE;
 			started = FALSE;
-			gnome_print_ps2_fprintf (ps2, "%g %g m" EOL, bpath->x3, bpath->y3);
+			gnome_print_ps2_print_double (ps2, "%g", bpath->x3);
+			gnome_print_ps2_fprintf (ps2, " ");
+			gnome_print_ps2_print_double (ps2, "%g", bpath->y3);
+			gnome_print_ps2_fprintf (ps2, " m" EOL);
 			break;
 		case ART_MOVETO:
 			if (started && closed)
 				gnome_print_ps2_fprintf (ps2, "h" EOL);
 			closed = TRUE;
 			started = TRUE;
-			gnome_print_ps2_fprintf (ps2, "%g %g m" EOL, bpath->x3, bpath->y3);
+			gnome_print_ps2_print_double (ps2, "%g", bpath->x3);
+			gnome_print_ps2_fprintf (ps2, " ");
+			gnome_print_ps2_print_double (ps2, "%g", bpath->y3);
+			gnome_print_ps2_fprintf (ps2, " m" EOL);
 			break;
 		case ART_LINETO:
-			gnome_print_ps2_fprintf (ps2, "%g %g l" EOL, bpath->x3, bpath->y3);
+			gnome_print_ps2_print_double (ps2, "%g", bpath->x3);
+			gnome_print_ps2_fprintf (ps2, " ");
+			gnome_print_ps2_print_double (ps2, "%g", bpath->y3);
+			gnome_print_ps2_fprintf (ps2, " l" EOL);
 			break;
 		case ART_CURVETO:
-			gnome_print_ps2_fprintf (ps2, "%g %g %g %g %g %g c" EOL,
-					bpath->x1, bpath->y1,
-					bpath->x2, bpath->y2,
-					bpath->x3, bpath->y3);
+			gnome_print_ps2_print_double (ps2, "%g", bpath->x1);
+			gnome_print_ps2_fprintf (ps2, " ");
+			gnome_print_ps2_print_double (ps2, "%g", bpath->y1);
+			gnome_print_ps2_fprintf (ps2, " ");
+			gnome_print_ps2_print_double (ps2, "%g", bpath->x2);
+			gnome_print_ps2_fprintf (ps2, " ");
+			gnome_print_ps2_print_double (ps2, "%g", bpath->y2);
+			gnome_print_ps2_fprintf (ps2, " ");
+			gnome_print_ps2_print_double (ps2, "%g", bpath->x3);
+			gnome_print_ps2_fprintf (ps2, " ");
+			gnome_print_ps2_print_double (ps2, "%g", bpath->y3);
+			gnome_print_ps2_fprintf (ps2, " c" EOL);
 			break;
 		default:
 			g_warning ("Path structure is corrupted");
@@ -956,17 +1179,16 @@ gnome_print_ps2_get_date (void)
 	return date;
 }
 
+
+/* Note "format" should be locale independent, so it should not use %g */
+/* and friends */
 static gint
 gnome_print_ps2_fprintf (GnomePrintPs2 *ps2, const char *format, ...)
 {
  	va_list arguments;
  	gchar *text;
-	gchar *oldlocale;
 	gint len;
 
-	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-		
  	va_start (arguments, format);
  	text = g_strdup_vprintf (format, arguments);
  	va_end (arguments);
@@ -974,9 +1196,21 @@ gnome_print_ps2_fprintf (GnomePrintPs2 *ps2, const char *format, ...)
 	len = fwrite (text, sizeof (gchar), strlen (text), ps2->buf);
 	g_free (text);
 
-	setlocale (LC_NUMERIC, oldlocale);
-	g_free (oldlocale);
-
 	return GNOME_PRINT_OK;
 }
 
+
+/* Allowed conversion specifiers are 'e', 'E', 'f', 'F', 'g' and 'G'. */
+static gint   
+gnome_print_ps2_print_double (GnomePrintPs2 *ps2, const gchar *format, gdouble x)
+{
+ 	gchar *text;
+
+	text = g_new (gchar, G_ASCII_DTOSTR_BUF_SIZE);
+	g_ascii_formatd (text, G_ASCII_DTOSTR_BUF_SIZE, format, x);
+
+	fwrite (text, sizeof (gchar), strlen (text), ps2->buf);
+	g_free (text);
+
+	return GNOME_PRINT_OK;
+}
