@@ -324,6 +324,7 @@ static void
 my_check_object (gint num, gint offset, guchar *real_in, GList *offsets, gint in_size)
 {
 	guchar *in = real_in;
+	guchar *end = in + in_size;
 	gchar *string;
 	gint len;
 	gchar *token;
@@ -343,7 +344,8 @@ my_check_object (gint num, gint offset, guchar *real_in, GList *offsets, gint in
 		gint guess = -1;
 		guess = my_guess_object_pos (in, num, in_size);
 		if (guess == -3)
-			return;
+			my_error ("Object %d is not located at offset %d, "
+				  "and could not be found\n", num, offset);
 		else
 			my_error ("Object %d is not located at offset %d, "
 				  "I think it is at %d\n",
@@ -384,14 +386,13 @@ my_check_object (gint num, gint offset, guchar *real_in, GList *offsets, gint in
 			if (debug > 1)
 				g_print ("[%s]\n", token);
 			if (strlen (token) == 0) {
-				g_print ("Len 0. Aborting!\n");
-				exit (1);
+				my_error ("Len 0. Aborting!\n");
 			}
 		}
 		if (strcmp (token, "/Length") == 0) {
 			g_free (token);
 			token = my_get_next_token (&in);
-//			g_print ("Size %s\n", token);
+/*			g_print ("Size %s\n", token); */
 			stream_len = atoi (token);
 			g_free (token);
 			token = my_get_next_token (&in);
@@ -403,7 +404,7 @@ my_check_object (gint num, gint offset, guchar *real_in, GList *offsets, gint in
 				/* steram_len contains the object num */
 				stream_len = my_get_size_from_object (real_in, stream_len, offsets);
 			}
-//			g_print ("Stream len is %d\n", stream_len);
+/*			g_print ("Stream len is %d\n", stream_len); */
 		}
 		/* We should recurse but i am lazy. Bah */
 		if (strncmp (token, "<<", 2) == 0) {
@@ -415,7 +416,7 @@ my_check_object (gint num, gint offset, guchar *real_in, GList *offsets, gint in
 			token = my_get_next_token (&in);
 		}
 		if (token [0] == '[') {
-//			g_print ("This is an array! -->%s<--\n", token);
+/*			g_print ("This is an array! -->%s<--\n", token); */
 			my_parse_array (token, &in);
 			token = g_strdup ("Dont' crash cause token was freed in parse array");
 		}
@@ -433,7 +434,8 @@ my_check_object (gint num, gint offset, guchar *real_in, GList *offsets, gint in
 		if (*in == '\n' || *in == '\r' || *in == 13) {
 			in++; a++;
 		}
-		if (strncmp (in + stream_len, "endstream", strlen ("endstream")) != 0) {
+		if (((in + stream_len + strlen ("endstream") > end))||
+		     (strncmp (in + stream_len, "endstream", strlen ("endstream")) != 0)) {
 			gint real_length;
 			if (debug) {
 				g_print ("We where checking in:\n");
@@ -469,6 +471,10 @@ my_check_xref (guchar *in, gint size)
 	offsets = my_get_offsets (in, size);
 	list = offsets;
 
+	if (!offsets) {
+		my_error ("Does not contain any objects");
+	}
+
 	while (list) {
 		my_check_object (object, GPOINTER_TO_INT (list->data), in, offsets, size);
 		object++;
